import 'package:barback/barback.dart';
import 'package:markdown/markdown.dart' as markdown;
import 'package:html/parser.dart' show parse;
import 'package:html/dom.dart';
import 'dart:async';
import 'dart:io';

// top level file bools
bool readmeProcessed = false;

class StaticTransformer extends Transformer {
  final BarbackSettings _settings;
  String get allowedExtensions => ".md .markdown .json";
  StaticTransformer.asPlugin(this._settings);

  Future apply(Transform transform) async {        
    // Looking for a config file
    if (transform.primaryInput.id.path.contains('config.json')) {
      transform.consumePrimary();
      
      // Use this as a cue to process the README.md
      if (readmeProcessed == false) {
        String readme = await new File('README.md').readAsString();  
        
        String license = await new File('LICENSE').readAsString();  
        
        Asset readmeAsset = new Asset.fromString(
            new AssetId(transform.primaryInput.id.package,'./web/index.html'), readme + 
            '\n----\n##LICENSE\n' + license);
        Asset transformedAsset = await processMarkdownAsset(readmeAsset);
        transform.addOutput(transformedAsset);
        readmeProcessed = true;
      }
      
      // Use this as a cue to process docs folder TODO
      
    }
    
    
    if (transform.primaryInput.id.extension == '.md' || transform.primaryInput.id.extension == '.markdown') {
      Asset transformedAsset = await processMarkdownAsset(transform.primaryInput);
      transform.addOutput(transformedAsset);
      transform.consumePrimary();
    }
  }
}

Future <Asset> processMarkdownAsset(Asset asset) async {
  String templateSrc = 
      await new File('packages/stoic/template.html').readAsString();
  Document document = 
      parse(templateSrc);
  String content = 
      await asset.readAsString(); 
  
  // change all paths TODO
  //content.replaceAll('web/', '');
  
  // add the content of the Markdown file
  document.querySelector('#content').innerHtml = markdown.markdownToHtml(content);
  
  // change the title
  document.querySelector('title').text = asset.id.package;
  
  // add the splash image
  if (getImagePath('splash') != null)
    document.querySelector('#splash').attributes['src'] = getImagePath('splash');
  
  // add the logo image
  if (getImagePath('logo') != null)
    document.querySelector('#logo').attributes['src'] = getImagePath('logo');
  
  // insert the navigation targets
  for (String page in getPages()) {
    document.querySelector('#navigation').append(
        new Element.tag('a')
          ..text = page
          ..attributes['href'] = '$page.html'              
    );
  }
  
  var id = asset.id.changeExtension('.html');
  return await new Asset.fromString(id, document.outerHtml);
}



getImagePath(String name) {
  List matches = new Directory('web').listSync(recursive:false).toList()
      .where((entity) => entity.path.contains('$name.'))
      .where((entity) => entity is File);
  if (matches.isEmpty) return null;
  
  File image = matches.first;  
  return image.path.split('/').last;
}


List <String> getPages() {
  List <String> pages = [];
  Directory docs = new Directory('docs');
  
  if (!docs.existsSync())
    return pages;
  
  new Directory('docs').listSync(recursive:false).toList()
   ..retainWhere((entity) => entity is File && (entity.path.endsWith('.md') || entity.path.endsWith('.markdown')))
   ..forEach((entity) => pages.add(
      entity.path.split('/').last.split('.')[0]
  ));
  
  return pages;
}